#include <cmath>

#include <initializer_list>
#include <iostream>
#include <map>
#include <stdexcept>
#include <string>

#include "Horner.h"

Polynomial::Polynomial(const std::initializer_list<std::map<unsigned, double>::value_type>& monomials) {
	for (std::map<unsigned, double>::value_type monomial : monomials) // "Range-based 'for'"
		_representation.insert(monomial);
}

double Polynomial::horner_method(const double x) const {
	double result = 0;
	// Existing (descending) order of powers of 'x':
	// for (std::map<unsigned, double>::const_reverse_iterator i = _representation.rbegin(); i != _representation.rend(); i++) std::cout << (*i).first << " - ";
	double coefficient;
	for (int n = (*_representation.rbegin()).first; n > 0; n--) {
		try {
			coefficient = _representation.at((unsigned)n); // '[]' conflicts with 'const' member function
			// std::cout << n << '\t';
		}
		catch (std::out_of_range& oor) {
			coefficient = 0.;
			// std::cerr << n << ": " << oor.what() << '\t';
		}
		result = (result + coefficient) * x;
	}
	try {
		result += _representation.at(0U);
	}
	catch (std::out_of_range& oor) {
		// std::cerr << "'x^^0' does not exist: " << oor.what() << '\t';
	}
	return result;
}

double Polynomial::simple_method(const double x) const {
	double result = 0;
	for (auto& [key, value] : _representation)
		result += value * ::pow(x, (double)key);
	return result;
}

std::string Polynomial::toString() const {
	std::string result;
	for (auto& [key, value] : _representation)
		result += std::to_string(value) + " * x^^" + std::to_string(key) + " + ";
	return result.substr(0, result.size() - 3);
}

int main(int argc, char** argv) {
	Polynomial p({ {4U,3.},{2U,2.},{0U,1.} });
	std::cout << p.toString() << std::endl;
	std::cout << "Horner method(2.): " << p.horner_method(2.) << std::endl << std::endl;;

	return 0;
}

